from wayfinder_paths.core.constants.base import (
    ADAPTER_BALANCE,
    ADAPTER_BRAP,
    ADAPTER_CCXT,
    ADAPTER_HYPERLEND,
    ADAPTER_HYPERLIQUID,
    ADAPTER_LEDGER,
    ADAPTER_MOONWELL,
    ADAPTER_POOL,
    ADAPTER_TOKEN,
    ADAPTER_UNISWAP,
    DEFAULT_HTTP_TIMEOUT,
    DEFAULT_NATIVE_GAS_UNITS,
    DEFAULT_PAGINATION_LIMIT,
    DEFAULT_SLIPPAGE,
    GAS_BUFFER_MULTIPLIER,
    MANTISSA,
    MAX_UINT256,
    ONE_GWEI,
    SECONDS_PER_YEAR,
    SUGGESTED_GAS_PRICE_MULTIPLIER,
    SUGGESTED_PRIORITY_FEE_MULTIPLIER,
)
from wayfinder_paths.core.constants.chains import (
    CHAIN_CODE_TO_ID,
    CHAIN_ID_ARBITRUM,
    CHAIN_ID_AVALANCHE,
    CHAIN_ID_BASE,
    CHAIN_ID_BSC,
    CHAIN_ID_ETHEREUM,
    CHAIN_ID_HYPEREVM,
    CHAIN_ID_PLASMA,
    CHAIN_ID_POLYGON,
    POA_MIDDLEWARE_CHAIN_IDS,
    PRE_EIP_1559_CHAIN_IDS,
    SUPPORTED_CHAINS,
)
from wayfinder_paths.core.constants.contracts import (
    ENSO_ROUTER,
    HYPE_FEE_WALLET,
    HYPE_OFT_ADDRESS,
    HYPERCORE_SENTINEL_ADDRESS,
    HYPEREVM_WHYPE,
    HYPERLEND_POOL_ADDRESSES_PROVIDER,
    HYPERLEND_UI_POOL_DATA_PROVIDER,
    KHYPE_ADDRESS,
    KHYPE_STAKING_ACCOUNTANT,
    LHYPE_ACCOUNTANT,
    LIFI_GENERIC,
    LIFI_ROUTER_HYPEREVM,
    LOOPED_HYPE_ADDRESS,
    MULTICALL3_ADDRESS,
    NATIVE_TOKEN_SENTINEL,
    UNISWAP_V3_FACTORY,
    UNISWAP_V3_NPM,
    ZERO_ADDRESS,
)

from .hyperliquid import (
    ARBITRUM_USDC_ADDRESS,
    ARBITRUM_USDC_TOKEN_ID,
    DEFAULT_HYPERLIQUID_BUILDER_FEE,
    DEFAULT_HYPERLIQUID_BUILDER_FEE_TENTHS_BP,
    HYPERLIQUID_BRIDGE_ADDRESS,
)

__all__ = [
    "NATIVE_TOKEN_SENTINEL",
    "ZERO_ADDRESS",
    "CHAIN_CODE_TO_ID",
    "DEFAULT_NATIVE_GAS_UNITS",
    "GAS_BUFFER_MULTIPLIER",
    "ONE_GWEI",
    "SUGGESTED_PRIORITY_FEE_MULTIPLIER",
    "SUGGESTED_GAS_PRICE_MULTIPLIER",
    "DEFAULT_SLIPPAGE",
    "DEFAULT_HTTP_TIMEOUT",
    "DEFAULT_PAGINATION_LIMIT",
    "MANTISSA",
    "SECONDS_PER_YEAR",
    "MAX_UINT256",
    "ADAPTER_BALANCE",
    "ADAPTER_BRAP",
    "ADAPTER_MOONWELL",
    "ADAPTER_HYPERLIQUID",
    "ADAPTER_POOL",
    "ADAPTER_TOKEN",
    "ADAPTER_LEDGER",
    "ADAPTER_HYPERLEND",
    "ADAPTER_UNISWAP",
    "ADAPTER_CCXT",
    "CHAIN_ID_ETHEREUM",
    "CHAIN_ID_BASE",
    "CHAIN_ID_ARBITRUM",
    "CHAIN_ID_BSC",
    "CHAIN_ID_POLYGON",
    "CHAIN_ID_AVALANCHE",
    "CHAIN_ID_PLASMA",
    "CHAIN_ID_HYPEREVM",
    "SUPPORTED_CHAINS",
    "POA_MIDDLEWARE_CHAIN_IDS",
    "PRE_EIP_1559_CHAIN_IDS",
    "HYPERLIQUID_BRIDGE_ADDRESS",
    "ARBITRUM_USDC_ADDRESS",
    "ARBITRUM_USDC_TOKEN_ID",
    "HYPE_FEE_WALLET",
    "DEFAULT_HYPERLIQUID_BUILDER_FEE_TENTHS_BP",
    "DEFAULT_HYPERLIQUID_BUILDER_FEE",
    "ENSO_ROUTER",
    "HYPE_OFT_ADDRESS",
    "HYPERCORE_SENTINEL_ADDRESS",
    "HYPEREVM_WHYPE",
    "HYPERLEND_POOL_ADDRESSES_PROVIDER",
    "HYPERLEND_UI_POOL_DATA_PROVIDER",
    "KHYPE_ADDRESS",
    "KHYPE_STAKING_ACCOUNTANT",
    "LHYPE_ACCOUNTANT",
    "LIFI_GENERIC",
    "LIFI_ROUTER_HYPEREVM",
    "LOOPED_HYPE_ADDRESS",
    "MULTICALL3_ADDRESS",
    "UNISWAP_V3_FACTORY",
    "UNISWAP_V3_NPM",
]
