TOKEN_ID_USDC_BASE = "usd-coin-base"
TOKEN_ID_WETH_BASE = "l2-standard-bridged-weth-base-base"
TOKEN_ID_WSTETH_BASE = "superbridge-bridged-wsteth-base-base"
TOKEN_ID_ETH_BASE = "ethereum-base"
TOKEN_ID_WELL_BASE = "moonwell-artemis-base"

TOKEN_ID_STETH = "staked-ether-ethereum"

TOKEN_ID_USDC_ARBITRUM = "usd-coin-arbitrum"
